-- drag_image.love

local gr, mg, ky, lm = love.graphics, love.image, love.keyboard, love.mouse

-- ************************
-- ***** control code *****
-- ************************
function love.load()
	screen_settings()
	load_images()
	end

function love.update(dt)
	mouseX, mouseY  = lm.getX(), lm.getY()
	check_image()
	if ky.isDown('s') then pic[num].alpha = pic[num].alpha < 255 and pic[num].alpha + .4 or 255 end
	if ky.isDown('a') then pic[num].alpha = pic[num].alpha >  10 and pic[num].alpha - .4 or  10 end
	if ky.isDown('q') then pic[num].size  = pic[num].size>0 and pic[num].size - 1 or 1 end
	if ky.isDown('w') then pic[num].size  = pic[num].size + 1
		if not image_on_screen(num) then pic[num].size = pic[num].size - 1 end
		end
	end

function love.draw()
	show_images()
	if show_text then add_text() end
	end

function love.keypressed( key )
	if key == 'escape' then love.event.push( 'quit' )	end
	if key == 'i' then show_text	= not show_text		end
	if key == 'b' then show_border	= not show_border	end
	if key == 'r' then reset()							end
	end

-- *****************************
-- ***** support functions *****
-- *****************************

function screen_settings()
	colorbg,	transbg	= { 0, 0, 0 }, 255
	mouseX,		mouseY	= 100, 100	-- arbitary numbers
	width,		height	= gr.getWidth(), gr.getHeight()
	selected_image	 	= 0		-- no image selected
	show_text			= true
	show_border			= true
	text			 	= {
						"(left click) selects image",
						"(left click) drag moves image",
						"(q/w) changes image size",
						"(a/s) changes transparency",
						"(b) show bounding box",
						"(r) reset images",
						"(i) for information",
						}
	gr.setFont( gr.newFont( 16 ) )
	gr.setCaption( "Image Dragging Test" )
	end

function load_images()
	path  = "images/"
	mig   = { "pic", "blue", "buda", "crider", "hand", "Bhutan" }
	ext	  = ".jpg"
	image = {}
	for i = 1, #mig do
		image[ i ] = gr.newImage( path..mig[i]..ext )
		end
	pic = {}
	for i = 1, #mig do
		pic[i] = {
			image	= image[i],
			x		= (i-1)*65+10,
			y		= (i-1)*60+30,
			alpha	= 185,
			drag	= false,
			size	= 150,
			width	= image[i]:getWidth(),
			height	= image[i]:getHeight()
			}
		end
	previous_mouseX	= {}	-- previous mouse position
	previous_mouseY = {}
	deltaX			= {}	-- change in mouse position
	deltaY			= {}
	restore			= {}	-- retain initial settings
	for i = 1, #pic do
		previous_mouseX[ i ]	= mouseX
		previous_mouseY[ i ]	= mouseY
		deltaX[ i ]				= 0
		deltaY[ i ] 			= 0
		restore[i]				= {}	-- back-up of initial settings
		for j, val in pairs( pic[i] ) do restore[i][j] = val end
		end
	end

function check_image()
	select_image()
	if selected_image > 0 then drag_image() end
	if not lm.isDown( 'l' ) or mouse_off_screen() then
		for i = 1, #pic do pic[ i ].drag = false end 	-- no activity
		selected_image = 0									-- nothing selected
		end
	end
function select_image()
	for i = #pic, 1, -1 do	--preference given to top most image
		if mouse_over_image( i ) then mouse_movement( i ) end
		if selected_image	== 0		and
			lm.isDown( 'l' )			and
			mouse_over_image( i ) 		then
				selected_image = i
				pic[#pic],pic[i] = pic[i],pic[#pic]	-- bring image to foreground
				num = #pic
				pic[ num ].drag = true
				break	-- insures ONLY 1 image active
			end
		end
	end
function drag_image()
	if image_on_screen( selected_image ) and mouse_over_image( selected_image ) then
		pic[ selected_image ].x = pic[ selected_image ].x + deltaX[ selected_image ]
		pic[ selected_image ].y = pic[ selected_image ].y + deltaY[ selected_image ]
		end
	end

function show_images()
	for i = 1, #pic do
		gr.setColor( 255, 255, 255, pic[i].alpha )
		gr.draw(
			pic[i].image,
			pic[i].x, pic[i].y,
			0,
			pic[i].size/pic[i].width, pic[i].size/pic[i].height
			)
		if show_border then
			gr.print(i,pic[i].x+5,pic[i].y+5)
			if i == num then gr.setColor(255,0,0,255) else gr.setColor(0,255,0,255) end
			gr.rectangle('line',pic[i].x,pic[i].y,pic[i].size,pic[i].size)
			end
		end
	end

function add_text()
	gr.setColor( 255, 255, 255 , 255 )
	msg = {}
	for i = 1, #pic do			-- status of each image
		if pic[i].drag then
			msg[i] = i.." drag"
			elseif
				mouse_over_image(i) then msg[i] = i.." over"
			else
				msg[i] = i.." clear"
			end
		gr.print(msg[i],80*(i-1)+10,height-20)
		end
	gr.setColor( 255, 255, 0, 255 )
	for i = 1, #text do
		gr.print( text[ i ], 265, 15 * i )
		end
	end

function mouse_movement( n )
	if not mouse_off_screen() then
		deltaX[ n ]				= mouseX - previous_mouseX[ n ]
		deltaY[ n ]				= mouseY - previous_mouseY[ n ]
		previous_mouseX[ n ]	= mouseX
		previous_mouseY[ n ]	= mouseY
		else
			deltaX[ n ], deltaY[ n ] = 0, 0	-- clear residule values
		end
	end

function image_on_screen( n )
	return 	pic[ n ].x + deltaX[ n ] > 0					and
			pic[ n ].x + pic[n].size + deltaX[n] < width	and
			pic[ n ].y + deltaY[ n ] > 0					and
			pic[ n ].y + pic[n].size + deltaY[n] < height
	end

function mouse_over_image( n )	-- needs to accomodate size change
	return  mouseX > pic[ n ].x and mouseX < pic[ n ].x + pic[ n ].size and
			mouseY > pic[ n ].y and mouseY < pic[ n ].y + pic[ n ].size
	end

function mouse_off_screen()
	return  mouseX < 0 or mouseX > width  or
			mouseY < 0 or mouseY > height
	end

function reset()
	for k = 1, #pic do
		for i, val in pairs( restore[k] ) do pic[k][i] = val end
		end
	end
